<?php
namespace Edubin\Navwalker;

if ( ! defined( 'ABSPATH' ) ) exit; // Exit if accessed directly
/**
 * WP Bootstrap Navwalker
 *
 * @package WP-Bootstrap-Navwalker
 */

/*
 * Class Name: Edubin_NavWalker
 * Plugin Name: WP Bootstrap Navwalker
 * Plugin URI:  https://github.com/wp-bootstrap/wp-bootstrap-navwalker
 * Description: A custom WordPress nav walker class to implement the Bootstrap 3 navigation style in a custom theme using the WordPress built in menu manager.
 * Author: Edward McIntyre - @twittem, WP Bootstrap
 * Version: 2.0.5
 * Author URI: https://github.com/wp-bootstrap
 * GitHub Plugin URI: https://github.com/wp-bootstrap/wp-bootstrap-navwalker
 * GitHub Branch: master
 * License: GPL-3.0+
 * License URI: http://www.gnu.org/licenses/gpl-3.0.txt
 */

/**
 * Edubin_NavWalker class.
 *
 * @extends Walker_Nav_Menu
 */
class Edubin_NavWalker extends \Walker_Nav_Menu {

	/**
	 * Start Level.
	 *
	 * @see Walker::start_lvl()
	 * @since 3.0.0
	 *
	 * @access public
	 * @param mixed $output Passed by reference. Used to append additional content.
	 * @param int   $depth (default: 0) Depth of page. Used for padding.
	 * @param array $args (default: array()) Arguments.
	 * @return void
	 */
	public function start_lvl( &$output, $depth = 0, $args = array() ) {
		$indent = str_repeat( "\t", $depth );
		$output .= "\n$indent<ul role=\"menu\" class=\" edubin-dropdown-menu\" >\n";
	}

	/**
	 * Start El.
	 *
	 * @see Walker::start_el()
	 * @since 3.0.0
	 *
	 * @access public
	 * @param mixed $output Passed by reference. Used to append additional content.
	 * @param mixed $item Menu item data object.
	 * @param int   $depth (default: 0) Depth of menu item. Used for padding.
	 * @param array $args (default: array()) Arguments.
	 * @param int   $id (default: 0) Menu item ID.
	 * @return void
	 */
	public function start_el( &$output, $item, $depth = 0, $args = array(), $id = 0 ) {
		$indent = ( $depth ) ? str_repeat( "\t", $depth ) : '';

		/**
		* Dividers, Headers or Disabled
		* =============================
		* Determine whether the item is a Divider, Header, Disabled or regular
		* menu item. To prevent errors we use the strcasecmp() function to so a
		* comparison that is not case sensitive. The strcasecmp() function returns
		* a 0 if the strings are equal.
	 	*/
		if ( 0 === strcasecmp( $item->attr_title, 'divider' ) && 1 === $depth ) {
			$output .= $indent . '<li role="presentation" class="divider">';
		} elseif ( 0 === strcasecmp( $item->title, 'divider' ) && 1 === $depth ) {
			$output .= $indent . '<li role="presentation" class="divider">';
		} elseif ( 0 === strcasecmp( $item->attr_title, 'dropdown-header' ) && 1 === $depth ) {
			$output .= $indent . '<li role="presentation" class="dropdown-header">' . esc_attr( $item->title );
		} elseif ( 0 === strcasecmp( $item->attr_title, 'disabled' ) ) {
			$output .= $indent . '<li role="presentation" class="disabled"><a href="#">' . esc_attr( $item->title ) . '</a>';
		} else {
			$class_names = $value = '';
			$classes = empty( $item->classes ) ? array() : (array) $item->classes;
			$classes[] = 'nav-item menu-item-' . $item->ID;
			$class_names = join( ' ', apply_filters( 'nav_menu_css_class', array_filter( $classes ), $item, $args ) );

			if ( $args->has_children && $depth === 0 ) :
				$class_names .= ' dropdown'; 
			elseif ( $args->has_children && $depth > 0 ) : 
				$class_names .= ' edubin-dropdown-submenu';
			endif;

			$edubin_mega_menu = $this->megaMenuAsSubMenu($item, $depth);
            if ( $edubin_mega_menu ) :
                $args->has_children = true;
                $class_names .= ' active-mega-menu menu-item-has-children dropdown';
			endif;
            if ( $item->edubin_menu_alignment ) :
                $class_names .= ' menu-align-' . $item->edubin_menu_alignment;
			endif;

			if ( in_array( 'current-menu-item', $classes, true ) ) :
				$class_names .= ' active'; 
			endif;
			$class_names = $class_names ? ' class="' . esc_attr( $class_names ) . '"' : '';
			$id = apply_filters( 'nav_menu_item_id', 'menu-item-' . $item->ID, $item, $args );
			$id = $id ? ' id="' . esc_attr( $id ) . '"' : '';
			$output .= $indent . '<li itemscope="itemscope" itemtype="https://www.schema.org/SiteNavigationElement"' . $id . $value . $class_names . '>';
			$atts = array();

			$atts['target'] = ! empty( $item->target )	? $item->target	: '';
			$atts['rel']    = ! empty( $item->xfn )		? $item->xfn	: '';
			// If item has_children add atts to a.
			if ( $args->has_children && 0 === $depth ) :
				$atts['href']           = ! empty( $item->url ) ? $item->url : '';
                // $atts['class']          = 'dropdown-toggle';
                $atts['aria-haspopup']  = 'true';
			else :
				$atts['href'] = ! empty( $item->url ) ? $item->url : '';
			endif;
			$atts = apply_filters( 'nav_menu_link_attributes', $atts, $item, $args );
			$attributes = '';
			foreach ( $atts as $attr => $value ) :
				if ( ! empty( $value ) ) :
					$value = ( 'href' === $attr ) ? esc_url( $value ) : esc_attr( $value );
					$attributes .= ' ' . $attr . '="' . $value . '"';
				endif;
			endforeach;
			$item_output = $args->before;
			// $item_output = '<span class="edubin-desktop-icon"><i class="flaticon-tumblr" aria-hidden="true"></i></span>';

			/*
			 * Glyphicons/Font-Awesome
			 * ===========
			 * Since the the menu item is NOT a Divider or Header we check the see
			 * if there is a value in the attr_title property. If the attr_title
			 * property is NOT null we apply it as the class name for the glyphicon.
			 */
			if ( ! empty( $item->attr_title ) ) :
							$pos = strpos( esc_attr( $item->attr_title ), 'glyphicon' );
				if ( false !== $pos ) :
					$item_output .= '<a' . $attributes . '><span class="glyphicon ' . esc_attr( $item->attr_title ) . '" aria-hidden="true"></span>&nbsp;';
							else :
								$item_output .= '<a' . $attributes . '><i class="fa ' . esc_attr( $item->attr_title ) . '" aria-hidden="true"></i>&nbsp;';
										endif;
			else :
				$item_output .= '<a class="nav-link"' . $attributes . '>';
			endif;
			$item_output .= $args->link_before . apply_filters( 'the_title', $item->title, $item->ID ) . $args->link_after;
			$item_output .= ( $args->has_children ) ? '<span class="edubin-menu-icon"><i class="flaticon-download" aria-hidden="true"></i></span></a>' : '</a>';
			$item_output .= $args->after;
			$item_output .= $this->displayMegaMenu( $item , $edubin_mega_menu );
			$output .= apply_filters( 'walker_nav_menu_start_el', $item_output, $item, $depth, $args );

		}
	}

	/**
	 * Traverse elements to create list from elements.
	 *
	 * Display one element if the element doesn't have any children otherwise,
	 * display the element and its children. Will only traverse up to the max
	 * depth and no ignore elements under that depth.
	 *
	 * This method shouldn't be called directly, use the walk() method instead.
	 *
	 * @see Walker::start_el()
	 * @since 2.5.0
	 *
	 * @access public
	 * @param mixed $element Data object.
	 * @param mixed $children_elements List of elements to continue traversing.
	 * @param mixed $max_depth Max depth to traverse.
	 * @param mixed $depth Depth of current element.
	 * @param mixed $args Arguments.
	 * @param mixed $output Passed by reference. Used to append additional content.
	 * @return null Null on failure with no changes to parameters.
	 */
	public function display_element( $element, &$children_elements, $max_depth, $depth, $args, &$output ) {
		if ( ! $element ) {
			return; }
		$id_field = $this->db_fields['id'];
		// Display this element.
		if ( is_object( $args[0] ) ) {
			$args[0]->has_children = ! empty( $children_elements[ $element->$id_field ] ); }
		parent::display_element( $element, $children_elements, $max_depth, $depth, $args, $output );
	}

	/**
	 * Menu Fallback
	 * =============
	 * If this function is assigned to the wp_nav_menu's fallback_cb variable
	 * and a menu has not been assigned to the theme location in the WordPress
	 * menu manager the function with display nothing to a non-logged in user,
	 * and will add a link to the WordPress menu manager if logged in as an admin.
	 *
	 * @param array $args passed from the wp_nav_menu function.
	 */
	public static function fallback( $args ) {
		if ( current_user_can( 'edit_theme_options' ) ) {

			/* Get Arguments. */
			$container = $args['container'];
			$container_id = $args['container_id'];
			$container_class = $args['container_class'];
			$menu_class = $args['menu_class'];
			$menu_id = $args['menu_id'];

			if ( $container ) {
				echo '<' . esc_attr( $container );
				if ( $container_id ) {
					echo ' id="' . esc_attr( $container_id ) . '"';
				}
				if ( $container_class ) {
					echo ' class="' . sanitize_html_class( $container_class ) . '"'; }
				echo '>';
			}
			echo '<ul';
			if ( $menu_id ) {
				echo ' id="' . esc_attr( $menu_id ) . '"'; }
			if ( $menu_class ) {
				echo ' class="' . esc_attr( $menu_class ) . '"'; }
			echo '>';
			echo '<li><a href="' . esc_url( admin_url( 'nav-menus.php' ) ) . '" title="' . __( 'Add a menu', 'edubin' ) . '">' . esc_attr( 'Add a menu', 'edubin' ) . '</a></li>';
			echo '</ul>';
			if ( $container ) {
				echo '</' . esc_attr( $container ) . '>'; }
		}
	}

	public function megaMenuAsSubMenu($item, $depth ) {
		return isset($item->edubin_mega_menu) && $item->edubin_mega_menu ? $item->edubin_mega_menu : false;
	}

	public function displayMegaMenu( $item, $edubin_mega_menu ) {
		if ( $edubin_mega_menu ) :
			$args = array(
				'name'        => $edubin_mega_menu,
				'post_type'   => 'edubin_megamenu',
				'post_status' => 'publish',
				'numberposts' => 1
			);
			$posts = get_posts($args);
			if ( $posts && isset( $posts[0] ) ) :
				$post = $posts[0];
				$content = apply_filters( 'edubin_generate_post_builder', do_shortcode( $post->post_content ), $post, $post->ID );
				$width = $item->edubin_menu_custom_width ? 'style="width:'.$item->edubin_menu_custom_width.'px"':"";

				if ( 'fullwidth' === $item->edubin_menu_alignment ) :
					$width = 'style="width:100%"';
				endif;
				
				return '<ul class="edubin-mega-menu ' . esc_attr( $post->post_name ) . '" ' . $width . '><div class="edubin-mega-menu-inner">' . $content . '</div></ul>';
			endif;
		endif;
		return '';
	}
}
